import sirv from "sirv";
import polka from "polka";
import compression from "compression";
import * as sapper from "@sapper/server";

import arangojs, { Database, aql } from "arangojs";
import { hash } from "bcrypt";
import { log } from "./logger";
import { Permissions } from "./lib/permissions";

const { PORT, NODE_ENV, DB_USERNAME, DB_PASSWORD, DB_URL, DB_NAME, API_KEY } =
	process.env;
const dev = NODE_ENV === "development";
const db = arangojs({
	auth: {
		username: DB_USERNAME || "root",
		password: DB_PASSWORD
	},
	url: DB_URL || "http://127.0.0.1:8529",
	databaseName: DB_NAME || "sbdatatracker"
});
const saltRounds = 10;
const delay = 3 * 60 * 1000;

const intervalFunc = async (db: Database) => {
	const cnf = db.collection("config");
	const users = db.collection("users");
	if (!(await cnf.exists())) {
		await cnf.create();
	}
	if (!(await users.exists())) {
		await users.create();
		await users.save({
			username: DB_USERNAME,
			password: await hash(DB_PASSWORD || "SBDataTracker", saltRounds),
			forceChangePassword: true,
			permissions: [Permissions.ALL]
		});
		await users.save({
			username: "_guest",
			permissions: [Permissions.VIEW_PROFILE]
		});
	}

	const cursor = await db.query(
		aql`
			FOR doc IN ${cnf}
				FILTER doc.type == "tracker"
				RETURN doc
		`,
		{
			count: true
		}
	);
	/*
		Instead of running a loop on all trackers at once,
		they are spread out evenly across the delay which
		lowers the chance of exceeding the Hypixel rate-limit
		when there are a lot of trackers.

		TODO : Add a hard-stop if the rate-limit is about
		to be exceeded.
		*/
	if (cursor.hasNext) {
		let counter = 0;
		let interval: NodeJS.Timeout;
		const iterate = async () => {
			if (++counter >= (cursor.count as number)) clearInterval(interval);

			const { _key, profile } = await cursor.next();
			if (profile) {
				const col = db.collection(`c${profile}`);
				if (!(await col.exists())) col.create();

				await col.save(await log(API_KEY || "", profile));
			} else {
				console.warn(
					`Configuration entry '${_key}' with type 'tracker' has no "profile" value`
				);
			}
		};
		// Separated into setInterval and a setTimeout so when the script is started,
		// immediately at least one profile is logged. This is useful for debugging
		// since you don't have to wait for up to 3 minutes to pass.
		if ((cursor.count as number) > 1)
			interval = setInterval(
				iterate,
				delay / ((cursor.count as number) - 1)
			);
		setTimeout(iterate);
	}
};

setInterval(intervalFunc, delay, db);
setTimeout(intervalFunc, 0, db);

polka({
	onError: err => {
		if (err) console.log("Error", err);
	}
})
	.use(
		compression({ threshold: 0 }),
		sirv("static", { dev }),
		sapper.middleware()
	)
	.listen(PORT);
